@@ -13,15 +13,15 @@
 
 import posixpath as pp
 import sys
-from warnings import warn
-
-from threading import local
+from abc import ABC, abstractmethod
 
 import numpy
 
 from .. import h5, h5s, h5t, h5r, h5d, h5p, h5fd, h5ds, _selector
-from ..h5py_warnings import H5pyDeprecationWarning
-from .base import HLObject, phil, with_phil, Empty, cached_property, find_item_type
+from .base import (
+    array_for_new_object, cached_property, Empty, find_item_type, HLObject,
+    phil, product, with_phil,
+)
 from . import filters
 from . import selections as sel
 from . import selections2 as sel2
@@ -38,14 +38,14 @@ def make_new_dset(parent, shape=None, dtype=None, data=None, name=None,
                   fletcher32=None, maxshape=None, compression_opts=None,
                   fillvalue=None, scaleoffset=None, track_times=False,
                   external=None, track_order=None, dcpl=None, dapl=None,
-                  efile_prefix=None, virtual_prefix=None,
-                  allow_unknown_filter=False):
+                  efile_prefix=None, virtual_prefix=None, allow_unknown_filter=False,
+                  rdcc_nslots=None, rdcc_nbytes=None, rdcc_w0=None, *,
+                  fill_time=None):
     """ Return a new low-level dataset identifier """
 
     # Convert data to a C-contiguous ndarray
     if data is not None and not isinstance(data, Empty):
-        from . import base
-        data = base.array_for_new_object(data, specified_dtype=dtype)
+        data = array_for_new_object(data, specified_dtype=dtype)
 
     # Validate shape
     if shape is None:
@@ -56,7 +56,7 @@ def make_new_dset(parent, shape=None, dtype=None, data=None, name=None,
         shape = data.shape
     else:
         shape = (shape,) if isinstance(shape, int) else tuple(shape)
-        if data is not None and (numpy.product(shape, dtype=numpy.ulonglong) != numpy.product(data.shape, dtype=numpy.ulonglong)):
+        if data is not None and (product(shape) != product(data.shape)):
             raise ValueError("Shape tuple is incompatible with data")
 
     if isinstance(maxshape, int):
@@ -106,7 +106,17 @@ def make_new_dset(parent, shape=None, dtype=None, data=None, name=None,
     dcpl = filters.fill_dcpl(
         dcpl or h5p.create(h5p.DATASET_CREATE), shape, dtype,
         chunks, compression, compression_opts, shuffle, fletcher32,
-        maxshape, scaleoffset, external, allow_unknown_filter)
+        maxshape, scaleoffset, external, allow_unknown_filter,
+        fill_time=fill_time)
+
+    # Check that compression roundtrips correctly if it was specified
+    if compression is not None:
+        if isinstance(compression, filters.FilterRefBase):
+            compression = compression.filter_id
+        if isinstance(compression, int):
+            compression = filters.get_filter_name(compression)
+        if compression not in filters.get_filters(dcpl):
+            raise ValueError(f'compression {compression!r} not in filters {filters.get_filters(dcpl)!r}')
 
     if fillvalue is not None:
         # prepare string-type dtypes for fillvalue
@@ -138,7 +148,7 @@ def make_new_dset(parent, shape=None, dtype=None, data=None, name=None,
     if maxshape is not None:
         maxshape = tuple(m if m is not None else h5s.UNLIMITED for m in maxshape)
 
-    if efile_prefix is not None or virtual_prefix is not None:
+    if any([efile_prefix, virtual_prefix, rdcc_nbytes, rdcc_nslots, rdcc_w0]):
         dapl = dapl or h5p.create(h5p.DATASET_ACCESS)
 
     if efile_prefix is not None:
@@ -147,12 +157,21 @@ def make_new_dset(parent, shape=None, dtype=None, data=None, name=None,
     if virtual_prefix is not None:
         dapl.set_virtual_prefix(virtual_prefix)
 
+    if rdcc_nbytes or rdcc_nslots or rdcc_w0:
+        cache_settings = list(dapl.get_chunk_cache())
+        if rdcc_nslots is not None:
+            cache_settings[0] = rdcc_nslots
+        if rdcc_nbytes is not None:
+            cache_settings[1] = rdcc_nbytes
+        if rdcc_w0 is not None:
+            cache_settings[2] = rdcc_w0
+        dapl.set_chunk_cache(*cache_settings)
+
     if isinstance(data, Empty):
         sid = h5s.create(h5s.NULL)
     else:
         sid = h5s.create_simple(shape, maxshape)
 
-
     dset_id = h5d.create(parent.id, name, tid, sid, dcpl=dcpl, dapl=dapl)
 
     if (data is not None) and (not isinstance(data, Empty)):
@@ -160,13 +179,13 @@ def make_new_dset(parent, shape=None, dtype=None, data=None, name=None,
 
     return dset_id
 
-def open_dset(parent, name, dapl=None, efile_prefix=None, virtual_prefix=None, **kwds):
+
+def open_dset(parent, name, dapl=None, efile_prefix=None, virtual_prefix=None,
+              rdcc_nslots=None, rdcc_nbytes=None, rdcc_w0=None, **kwds):
     """ Return an existing low-level dataset identifier """
 
-    if efile_prefix is not None or virtual_prefix is not None:
+    if any([efile_prefix, virtual_prefix, rdcc_nbytes, rdcc_nslots, rdcc_w0]):
         dapl = dapl or h5p.create(h5p.DATASET_ACCESS)
-    else:
-        dapl = dapl or None
 
     if efile_prefix is not None:
         dapl.set_efile_prefix(efile_prefix)
@@ -174,53 +193,93 @@ def open_dset(parent, name, dapl=None, efile_prefix=None, virtual_prefix=None, *
     if virtual_prefix is not None:
         dapl.set_virtual_prefix(virtual_prefix)
 
+    if rdcc_nbytes or rdcc_nslots or rdcc_w0:
+        cache_settings = list(dapl.get_chunk_cache())
+        if rdcc_nslots is not None:
+            cache_settings[0] = rdcc_nslots
+        if rdcc_nbytes is not None:
+            cache_settings[1] = rdcc_nbytes
+        if rdcc_w0 is not None:
+            cache_settings[2] = rdcc_w0
+        dapl.set_chunk_cache(*cache_settings)
+
     dset_id = h5d.open(parent.id, name, dapl=dapl)
 
     return dset_id
 
-class AstypeWrapper:
+
+
+class AbstractView(ABC):
+    _dset: "Dataset"
+
+    def __init__(self, dset):
+        self._dset = dset
+
+    def __len__(self):
+        return len(self._dset)
+
+    @property
+    @abstractmethod
+    def dtype(self):
+        ...  # pragma: nocover
+
+    @property
+    def ndim(self):
+        return self._dset.ndim
+
+    @property
+    def shape(self):
+        return self._dset.shape
+
+    @property
+    def size(self):
+        return self._dset.size
+
+    @abstractmethod
+    def __getitem__(self, idx):
+        ...  # pragma: nocover
+
+    def __array__(self, dtype=None, copy=None):
+        if copy is False:
+            raise ValueError(
+                f"{self.__class__.__name__}.__array__ received {copy=} "
+                "but memory allocation cannot be avoided on read"
+            )
+
+        # If self.ndim == 0, convert np.generic back to np.ndarray
+        return numpy.asarray(self[()], dtype=dtype or self.dtype)
+
+class AsTypeView(AbstractView):
     """Wrapper to convert data on reading from a dataset.
     """
     def __init__(self, dset, dtype):
-        self._dset = dset
+        super().__init__(dset)
         self._dtype = numpy.dtype(dtype)
 
-    def __getitem__(self, args):
-        return self._dset.__getitem__(args, new_dtype=self._dtype)
+    @property
+    def dtype(self):
+        return self._dtype
 
-    def __enter__(self):
-        # pylint: disable=protected-access
-        warn(
-            "Using astype() as a context manager is deprecated. "
-            "Slice the returned object instead, like: ds.astype(np.int32)[:10]",
-            category=H5pyDeprecationWarning, stacklevel=2,
-        )
-        self._dset._local.astype = self._dtype
-        return self
+    def __getitem__(self, idx):
+        return self._dset.__getitem__(idx, new_dtype=self._dtype)
 
-    def __exit__(self, *args):
-        # pylint: disable=protected-access
-        self._dset._local.astype = None
-
-    def __len__(self):
-        """ Get the length of the underlying dataset
-
-        >>> length = len(dataset.astype('f8'))
-        """
-        return len(self._dset)
+    def __array__(self, dtype=None, copy=None):
+        return self._dset.__array__(dtype or self._dtype, copy)
 
 
-class AsStrWrapper:
+class AsStrView(AbstractView):
     """Wrapper to decode strings on reading the dataset"""
     def __init__(self, dset, encoding, errors='strict'):
-        self._dset = dset
-        if encoding is None:
-            encoding = h5t.check_string_dtype(dset.dtype).encoding
+        super().__init__(dset)
         self.encoding = encoding
         self.errors = errors
 
-    def __getitem__(self, args):
-        bytes_arr = self._dset[args]
+    @property
+    def dtype(self):
+        return numpy.dtype(object)
+
+    def __getitem__(self, idx):
+        bytes_arr = self._dset[idx]
         # numpy.char.decode() seems like the obvious thing to use. But it only
         # accepts numpy string arrays, not object arrays of bytes (which we
         # return from HDF5 variable-length strings). And the numpy
@@ -234,38 +293,32 @@ class AsStrWrapper:
             b.decode(self.encoding, self.errors) for b in bytes_arr.flat
         ], dtype=object).reshape(bytes_arr.shape)
 
-    def __len__(self):
-        """ Get the length of the underlying dataset
 
-        >>> length = len(dataset.asstr())
-        """
-        return len(self._dset)
-
-
-class FieldsWrapper:
+class FieldsView(AbstractView):
     """Wrapper to extract named fields from a dataset with a struct dtype"""
-    extract_field = None
 
     def __init__(self, dset, prior_dtype, names):
-        self._dset = dset
+        super().__init__(dset)
         if isinstance(names, str):
             self.extract_field = names
             names = [names]
+        else:
+            self.extract_field = None
         self.read_dtype = readtime_dtype(prior_dtype, names)
 
-    def __getitem__(self, args):
-        data = self._dset.__getitem__(args, new_dtype=self.read_dtype)
+    @property
+    def dtype(self):
+        t = self.read_dtype
+        if self.extract_field is not None:
+            t = t[self.extract_field]
+        return t
+
+    def __getitem__(self, idx):
+        data = self._dset.__getitem__(idx, new_dtype=self.read_dtype)
         if self.extract_field is not None:
             data = data[self.extract_field]
         return data
 
-    def __len__(self):
-        """ Get the length of the underlying dataset
-
-        >>> length = len(dataset.fields(['x', 'y']))
-        """
-        return len(self._dset)
-
 
 def readtime_dtype(basetype, names):
     """Make a NumPy compound dtype with a subset of available fields"""
@@ -273,7 +326,7 @@ def readtime_dtype(basetype, names):
         raise ValueError("Field names only allowed for compound types")
 
     for name in names:  # Check all names are legal
-        if not name in basetype.names:
+        if name not in basetype.names:
             raise ValueError("Field %s does not appear in this type." % name)
 
     return numpy.dtype([(name, basetype.fields[name][0]) for name in names])
@@ -313,10 +366,10 @@ class ChunkIterator:
         self._layout = dset.chunks
         if source_sel is None:
             # select over entire dataset
-            slices = []
-            for dim in range(rank):
-                slices.append(slice(0, self._shape[dim]))
-            self._sel = tuple(slices)
+            self._sel = tuple(
+                slice(0, self._shape[dim])
+                for dim in range(rank)
+            )
         else:
             if isinstance(source_sel, slice):
                 self._sel = (source_sel,)
@@ -367,10 +420,11 @@ class ChunkIterator:
 
             if dim > 0:
                 # reset to the start and continue iterating with higher dimension
-                self._chunk_index[dim] = 0
+                self._chunk_index[dim] = s.start // self._layout[dim]
             dim -= 1
         return tuple(slices)
 
+
 class Dataset(HLObject):
 
     """
@@ -383,7 +437,7 @@ class Dataset(HLObject):
 
         >>> double_precision = dataset.astype('f8')[0:100:2]
         """
-        return AstypeWrapper(self, dtype)
+        return AsTypeView(self, dtype)
 
     def asstr(self, encoding=None, errors='strict'):
         """Get a wrapper to read string data as Python strings:
@@ -402,7 +456,7 @@ class Dataset(HLObject):
             )
         if encoding is None:
             encoding = string_info.encoding
-        return AsStrWrapper(self, encoding, errors=errors)
+        return AsStrView(self, encoding, errors=errors)
 
     def fields(self, names, *, _prior_dtype=None):
         """Get a wrapper to read a subset of fields from a compound data type:
@@ -415,7 +469,7 @@ class Dataset(HLObject):
         """
         if _prior_dtype is None:
             _prior_dtype = self.dtype
-        return FieldsWrapper(self, _prior_dtype, names)
+        return FieldsView(self, _prior_dtype, names)
 
     if MPI:
         @property
@@ -467,7 +521,7 @@ class Dataset(HLObject):
         if self._is_empty:
             size = None
         else:
-            size = numpy.prod(self.shape, dtype=numpy.intp)
+            size = product(self.shape)
 
         # If the file is read-only, cache the size to speed-up future uses.
         # This cache is invalidated by .refresh() when using SWMR.
@@ -611,6 +665,20 @@ class Dataset(HLObject):
         """Check if extent type is empty"""
         return self._extent_type == h5s.NULL
 
+    @cached_property
+    def _dcpl(self):
+        """
+        The dataset creation property list used when this dataset was created.
+        """
+        return self.id.get_create_plist()
+
+    @cached_property
+    def _filters(self):
+        """
+        The active filters of the dataset.
+        """
+        return filters.get_filters(self._dcpl)
+
     @with_phil
     def __init__(self, bind, *, readonly=False):
         """ Create a new Dataset object by binding to a low-level DatasetID.
@@ -619,13 +687,9 @@ class Dataset(HLObject):
             raise ValueError("%s is not a DatasetID" % bind)
         super().__init__(bind)
 
-        self._dcpl = self.id.get_create_plist()
         self._dxpl = h5p.create(h5p.DATASET_XFER)
-        self._filters = filters.get_filters(self._dcpl)
         self._readonly = readonly
         self._cache_props = {}
-        self._local = local()
-        self._local.astype = None
 
     def resize(self, size, axis=None):
         """ Resize the dataset, or the specified axis.
@@ -733,9 +797,6 @@ class Dataset(HLObject):
         """
         args = args if isinstance(args, tuple) else (args,)
 
-        if new_dtype is None:
-            new_dtype = getattr(self._local, 'astype', None)
-
         if self._fast_read_ok and (new_dtype is None):
             try:
                 return self._fast_reader.read(args)
@@ -848,17 +909,18 @@ class Dataset(HLObject):
         if vlen is not None and vlen not in (bytes, str):
             try:
                 val = numpy.asarray(val, dtype=vlen)
-            except ValueError:
+            except (ValueError, TypeError):
                 try:
                     val = numpy.array([numpy.array(x, dtype=vlen)
                                        for x in val], dtype=self.dtype)
-                except ValueError:
+                except (ValueError, TypeError):
                     pass
             if vlen == val.dtype:
                 if val.ndim > 1:
                     tmp = numpy.empty(shape=val.shape[:-1], dtype=object)
                     tmp.ravel()[:] = [i for i in val.reshape(
-                        (numpy.product(val.shape[:-1], dtype=numpy.ulonglong), val.shape[-1]))]
+                        (product(val.shape[:-1]), val.shape[-1])
+                    )]
                 else:
                     tmp = numpy.array([None], dtype=object)
                     tmp[0] = val
@@ -967,8 +1029,7 @@ class Dataset(HLObject):
         if mshape == () and selection.array_shape != ():
             if self.dtype.subdtype is not None:
                 raise TypeError("Scalar broadcasting is not supported for array dtypes")
-            if self.chunks and (numpy.prod(self.chunks, dtype=numpy.float64) >=
-                                numpy.prod(selection.array_shape, dtype=numpy.float64)):
+            if self.chunks and (product(self.chunks) >= product(selection.array_shape)):
                 val2 = numpy.empty(selection.array_shape, dtype=val.dtype)
             else:
                 val2 = numpy.empty(selection.array_shape[-1], dtype=val.dtype)
@@ -1032,15 +1093,20 @@ class Dataset(HLObject):
                 self.id.write(mspace, fspace, source, dxpl=self._dxpl)
 
     @with_phil
-    def __array__(self, dtype=None):
+    def __array__(self, dtype=None, copy=None):
         """ Create a Numpy array containing the whole dataset.  DON'T THINK
         THIS MEANS DATASETS ARE INTERCHANGEABLE WITH ARRAYS.  For one thing,
         you have to read the whole dataset every time this method is called.
         """
+        if copy is False:
+            raise ValueError(
+                f"Dataset.__array__ received {copy=} "
+                "but memory allocation cannot be avoided on read"
+            )
         arr = numpy.zeros(self.shape, dtype=self.dtype if dtype is None else dtype)
 
         # Special case for (0,)*-shape datasets
-        if numpy.product(self.shape, dtype=numpy.ulonglong) == 0:
+        if self.size == 0:
             return arr
 
         self.read_direct(arr)
@@ -1114,3 +1180,12 @@ class Dataset(HLObject):
         You can optionally pass a name to associate with this scale.
         """
         h5ds.set_scale(self._id, self._e(name))
+
+    @property
+    @with_phil
+    def is_scale(self):
+        """Return ``True`` if this dataset is also a dimension scale.
+
+        Return ``False`` otherwise.
+        """
+        return h5ds.is_scale(self._id)
